use crate::prelude::*;

use anyhow::Result;
use cgmath::Rad;
use std::sync::Arc;
use utilities::{impl_reprc, prelude::cgmath::Vector3};

impl_reprc!(
    #[derive(Debug)]
    pub struct LightData {
        #[assume_reprc]
        position: Vector3<f32>,
        #[assume_reprc]
        power: f32,
        #[assume_reprc]
        color: Vector3<f32>,
        #[assume_reprc]
        arc: Rad<f32>, // Rad
        #[assume_reprc]
        direction: Vector3<f32>,
        #[assume_reprc]
        t_min: f32,
    }
);

impl From<&Light> for LightData {
    fn from(light: &Light) -> Self {
        Self {
            position: light.position(),
            power: light.power(),
            color: light.color(),
            arc: light.angle(),
            direction: light.direction(),
            t_min: light.t_min(),
        }
    }
}

pub struct RayTracerLight {
    light: Light,

    buffer: Arc<Buffer<LightData>>,
}

impl RayTracerLight {
    pub fn new(device: &Arc<Device>, light: Light) -> Result<Self> {
        Ok(Self {
            buffer: Buffer::builder()
                .set_usage(VK_BUFFER_USAGE_STORAGE_BUFFER_BIT)
                .set_memory_usage(MemoryUsage::CpuOnly)
                .set_data(&[LightData::from(&light)])
                .build(device.clone())?,
            light,
        })
    }

    pub fn buffer(&self) -> &Arc<Buffer<LightData>> {
        &self.buffer
    }

    pub fn eq(&self, light: &Light) -> bool {
        self.light.eq(light)
    }
}
