#version 450

// input parameter
layout (location = 0) in vec3 in_world_pos;

#ifdef COLOR_UVS
    layout (location = 1) in vec2 in_uv;
#endif

#ifdef NORMALS
    #ifdef NORMAL_UVS
        layout (location = 2) in vec3 in_tangent;
    #endif

    layout (location = 3) in vec3 in_normal;
#endif

// output parameter
layout (location = 0) out vec4 out_position;
layout (location = 1) out vec4 out_normal;
layout (location = 2) out vec4 out_albedo;

layout (set = 0, binding = 1) uniform Material {
    vec4 color;

    int color_texture_index;
    int normal_texture_index;
    int emissive_texture_index;
    int roughness_texture_index;

    float metallic_factor;
    vec3 emissive_factor;

    float roughness_factor;
} material;

// descriptors
#ifdef COLOR_UVS
	layout (set = 0, binding = 2) uniform sampler2D color_texture;
#endif

#ifdef NORMAL_UVS
	layout (set = 0, binding = 4) uniform sampler2D normal_texture;
#endif

void main()
{
    out_position = vec4(in_world_pos, material.metallic_factor);

    #ifdef NORMALS
        vec3 N = normalize(in_normal);

        #ifdef NORMAL_UVS
            N.y = -N.y;
            vec3 T = normalize(in_tangent);
            vec3 B = cross(N, T);
            mat3 TBN = mat3(T, B, N);
            vec3 tnorm = TBN *
                normalize(texture(normal_texture, in_uv).xyz * 2.0 - vec3(1.0));

            out_normal = vec4(tnorm, 1.0);
        #else
            out_normal = vec4(N, 1.0);
        #endif
    #else
        out_normal = vec4(0.0, 0.0, 1.0, 1.0);
    #endif

    #ifdef COLOR_UVS
        out_albedo = texture(color_texture, in_uv);
    #else
        out_albedo = material.color;
    #endif
}
