use anyhow::Result;
use engine::prelude::*;

use std::time::Duration;

use crate::Game;

#[derive(Serialize, Deserialize)]
pub struct Ghost {
    start: Duration,

    duration: Duration,
    start_height: f32,
    end_height: f32,
}

impl Ghost {
    pub fn new(
        game: &Game,
        location: &Location,
        radius: f32,
        height: f32,
        start: Duration,
    ) -> Result<EntityObject> {
        let mut ghost = game
            .entity_manager()
            .load_entity(game.engine().assets(), "Ghost")?;

        let ghost_location = Location::new_and_setup(&mut ghost)?;
        ghost_location.update(location.position(), location.direction());
        ghost_location.set_scale_uniform(radius * 2.0);

        let start_height = ghost_location.position().z + height;
        let end_height = start_height + 2.0 * radius;

        let me = Ghost {
            start,

            duration: Duration::from_secs(2),
            start_height,
            end_height,
        };

        ghost.insert_component(me);

        Ok(ghost)
    }

    pub fn update(&self, now: Duration, location: &mut Location) -> Result<bool> {
        if self.start + self.duration < now {
            return Ok(false);
        }

        let diff = now - self.start;
        let ratio = diff.as_secs_f32() / self.duration.as_secs_f32();

        let final_height = self.start_height + self.end_height * ratio;

        let mut position = location.position();
        position.z = final_height;

        location.set_position(position);

        Ok(true)
    }
}

impl EntityComponent for Ghost {
    fn name(&self) -> &str {
        Self::debug_name()
    }
}

impl ComponentDebug for Ghost {
    fn debug_name() -> &'static str {
        "Ghost"
    }
}
